=== modified file 'actions.yaml'
--- actions.yaml	2015-04-07 13:58:41 +0000
+++ actions.yaml	2015-09-14 20:23:58 +0000
@@ -1,2 +1,4 @@
 git-reinstall:
   description: Reinstall openstack-dashboard from the openstack-origin-git repositories.
+openstack-upgrade:
+  description: Perform openstack upgrades. Config option action-managed-upgrade must be set to True.

=== added symlink 'actions/openstack-upgrade'
=== target is 'openstack_upgrade.py'
=== added file 'actions/openstack_upgrade.py'
--- actions/openstack_upgrade.py	1970-01-01 00:00:00 +0000
+++ actions/openstack_upgrade.py	2015-09-14 20:23:58 +0000
@@ -0,0 +1,32 @@
+#!/usr/bin/python
+import sys
+
+sys.path.append('hooks/')
+
+from charmhelpers.contrib.openstack.utils import (
+    do_action_openstack_upgrade,
+)
+
+from horizon_utils import (
+    do_openstack_upgrade,
+    register_configs
+)
+
+
+CONFIGS = register_configs()
+
+
+def openstack_upgrade():
+    """Upgrade packages to config-set Openstack version.
+
+    If the charm was installed from source we cannot upgrade it.
+    For backwards compatibility a config flag must be set for this
+    code to run, otherwise a full service level upgrade will fire
+    on config-changed."""
+
+    do_action_openstack_upgrade('openstack-dashboard',
+                                do_openstack_upgrade,
+                                CONFIGS)
+
+if __name__ == '__main__':
+    openstack_upgrade()

=== modified file 'config.yaml'
--- config.yaml	2015-07-10 14:18:48 +0000
+++ config.yaml	2015-09-14 20:23:58 +0000
@@ -175,4 +175,13 @@
     description: |
       A comma-separated list of nagios servicegroups. If left empty, the
       nagios_context will be used as the servicegroup.
-
+  action-managed-upgrade:
+    type: boolean
+    default: False
+    description: |
+      If True enables openstack upgrades for this charm via juju actions.
+      You will still need to set openstack-origin to the new repository but
+      instead of an upgrade running automatically across all units, it will
+      wait for you to execute the openstack-upgrade action for this charm on
+      each unit. If False it will revert to existing behavior of upgrading
+      all units on config change.

=== modified file 'hooks/charmhelpers/contrib/openstack/utils.py'
--- hooks/charmhelpers/contrib/openstack/utils.py	2015-09-04 13:59:26 +0000
+++ hooks/charmhelpers/contrib/openstack/utils.py	2015-09-14 20:23:58 +0000
@@ -25,6 +25,7 @@
 import re
 
 import six
+import traceback
 import yaml
 
 from charmhelpers.contrib.network import ip
@@ -34,6 +35,8 @@
 )
 
 from charmhelpers.core.hookenv import (
+    action_fail,
+    action_set,
     config,
     log as juju_log,
     charm_dir,
@@ -114,6 +117,7 @@
     ('2.2.1', 'kilo'),
     ('2.2.2', 'kilo'),
     ('2.3.0', 'liberty'),
+    ('2.4.0', 'liberty'),
 ])
 
 # >= Liberty version->codename mapping
@@ -748,3 +752,47 @@
         return projects[key]
 
     return None
+
+
+def do_action_openstack_upgrade(package, upgrade_callback, configs):
+    """Perform action-managed OpenStack upgrade.
+
+    Upgrades packages to the configured openstack-origin version and sets
+    the corresponding action status as a result.
+
+    If the charm was installed from source we cannot upgrade it.
+    For backwards compatibility a config flag (action-managed-upgrade) must
+    be set for this code to run, otherwise a full service level upgrade will
+    fire on config-changed.
+
+    @param package: package name for determining if upgrade available
+    @param upgrade_callback: function callback to charm's upgrade function
+    @param configs: templating object derived from OSConfigRenderer class
+
+    @return: True if upgrade successful; False if upgrade failed or skipped
+    """
+    ret = False
+
+    if git_install_requested():
+        action_set({'outcome': 'installed from source, skipped upgrade.'})
+    else:
+        if openstack_upgrade_available(package):
+            if config('action-managed-upgrade'):
+                juju_log('Upgrading OpenStack release')
+
+                try:
+                    upgrade_callback(configs=configs)
+                    action_set({'outcome': 'success, upgrade completed.'})
+                    ret = True
+                except:
+                    action_set({'outcome': 'upgrade failed, see traceback.'})
+                    action_set({'traceback': traceback.format_exc()})
+                    action_fail('do_openstack_upgrade resulted in an '
+                                'unexpected error')
+            else:
+                action_set({'outcome': 'action-managed-upgrade config is '
+                                       'False, skipped upgrade.'})
+        else:
+            action_set({'outcome': 'no upgrade available.'})
+
+    return ret

=== modified file 'hooks/horizon_hooks.py'
--- hooks/horizon_hooks.py	2015-05-07 14:33:05 +0000
+++ hooks/horizon_hooks.py	2015-09-14 20:23:58 +0000
@@ -104,7 +104,7 @@
     if git_install_requested():
         if config_value_changed('openstack-origin-git'):
             git_install(config('openstack-origin-git'))
-    else:
+    elif not config('action-managed-upgrade'):
         if openstack_upgrade_available('openstack-dashboard'):
             do_openstack_upgrade(configs=CONFIGS)
 

